#!/usr/bin/env lua
allocator = require "allocator"

-- where we'll store our info
students = {}
subjects = {}
priorities    = {}

-- make them behave a bit like objects
setmetatable(students, {__index=table})
setmetatable(subjects, {__index=table})
setmetatable(priorities,    {__index=table})

-- functions that will be available to the config file
function Subject(name)
    subjects:insert(name)
    return #subjects
end

function Student(name, ...)
    students:insert(name)
    prio_list = {...}
    prios = {}
    for i,p in ipairs(prio_list) do
        prios[p] = math.max(#subjects-(i-1), 0)
    end
    priorities:insert(prios)
end

-- load and execute config file
generate_model = assert(loadfile(arg[1], "bt", {Student=Student, Subject=Subject}))
generate_model()

-- call allocator and display solution
solution, optimal = allocator.allocate(#students,#subjects,table.unpack(priorities))
print("Solution:", optimal and "(optimal)")
print("---------")
for i,s in ipairs(students) do
    print(s, " → ", subjects[solution[i]] or "(NO REMAINING SUBJECT IN PRIORITY LIST)")
end
